;;; "clp-ga-crossing-over.scm"
;(require (lib "breakpoint.ss"))

; Das crossing-over ist sehr viel schwerer zu realisieren.
; Eine Moeglichkeit besteht darin, mehrere Container aus einem
; Individuum mit denen aus einem anderen dann auszutauschen,
; wenn dabei jeweils zulaessige Loesungen entstehen.
; (--> "lebensfaehig")
;;; ----- crossing-over -----
; steuert das crossing-over ueber den pool
(define
  (crossing-over anzahl pool akku)
  (cond
    ((zero? anzahl) (append pool akku))
    (else
     (crossing-over 
      (sub1 anzahl)
      pool
      (append
        (cross
         (random (length (car pool)))
         (random (length (car pool)))
         (list-ref pool (random (length pool)))
         (list-ref pool (random (length pool))))
         akku)))))

;;; ----- cross -----
; Vorbedingung: von >= 1  und bis <= Anzahl Container
(define
  (cross von bis liste-1 liste-2)
  (append
   (lebensfaehig
    *Stuecke* ; alle Stuecke
    (fuelle-um
     von
     bis
     (cdr liste-1)  ; nicht eingefuellte abschneiden
     (cdr liste-2))) ; nicht eingefuellte abschneiden
   (lebensfaehig
    *Stuecke*
    (fuelle-um
     von
     bis
     (cdr liste-2) ; nicht eingefuellte abschneiden
     (cdr liste-1))))) ; nicht eingefuellte abschneiden

;;; ----- fuelle-um -----
; fuellt mit den Containern der jeweils anderen Liste
(define
  (fuelle-um von bis liste-1 liste-2)
  (cond
    ((null? liste-1) liste-1)
    ((> von 1)      ; vor den auszutauschenden Containern
     (cons
      (car liste-1)
      (fuelle-um (sub1 von) (sub1 bis) (cdr liste-1) (cdr liste-2))))
    ((< bis 1)      ; nach den auszutauschenden Containern
     liste-1)
    (else            ; hier wird ausgetauscht
     (cons
      (car liste-2)
      (fuelle-um (sub1 von) (sub1 bis) (cdr liste-1) (cdr liste-2))))))

;;; ----- lebensfaehig -----
; Testet, ob die Loesung zulaessig ist, also nur vorhandene Stuecke eingeladen wurden.
; Wenn nein, gibt die Funktion die leere Liste zurueck,
; sonst die richtige Containerliste mit der Liste der nicht eingefuellten davor.
(define
  (lebensfaehig stuecke containerliste)
  (let loop
    ((verbleibende stuecke)
     (eingefuellte ; alle in den Containern enthaltenen Stuecke
      (apply append (map cdr (cdr containerliste)))))
    (cond
      ((null? eingefuellte) ; alles OK
       (list (cons (cons 0 verbleibende) containerliste)))
      ((null? verbleibende) '()) ; geht nicht !
      ((member (car eingefuellte) verbleibende)
       (loop
        (entferne-aus (car eingefuellte) verbleibende)
        (cdr eingefuellte)))
      (else '()))))

;;; ----- entferne-aus -----
; Hilfsfunktion
(define
  (entferne-aus element liste)
  (cond
    ((null? liste) (error "kann nicht sein!"))
    ((equal? (car liste) element) (cdr liste))
    (else
     (cons (car liste) (entferne-aus element (cdr liste))))))
