;;; "clp-ga-mutation.scm"

;;; ===== Mutation =====
; Bei der Mutation wird ein zufaellig gewaehltes Stueck
; aus einem zufaellig gewaehlten Container entnommen und
; in einen zufaellig gewaehlten anderen getan.
(define
  (Mutation pool akku)
  (cond
    ((null? pool) akku)
    (else
     (Mutation
      (cdr pool)
      (cons
        (mutiere
         (random (length (car pool))) 
         (random (length (car pool)))
         (car pool))
        (cons
         (car pool)
         akku))))))

;;; ----- mutiere -----
; Fuehrt die Mutation durch
; zahl x zahl x liste --> liste
(define
  (mutiere von-container-nr nach-container-nr container-liste)
  ;(bkpt  von-container-nr nach-container-nr container-liste)
  (cond
   ((= von-container-nr nach-container-nr)
    container-liste) ; sinnlos!
   ((leer? von-container-nr container-liste)
    container-liste) ; geht nicht!
   (else
    (letrec
        ((container (list-ref container-liste von-container-nr))
         (stueck-nr (random (length container)))
         (stueck (list-ref container stueck-nr)))
      (entferne
       stueck
       von-container-nr
       (fuelle
        stueck
        nach-container-nr
        container-liste))))))
     
;;; ----- leer? -----
; prueft, ob der container mit der uebergebenen nummer leer ist.
; nr: 0 .. n-1
(define
  (leer? nr container-liste)
  (null? (cdr (list-ref container-liste nr))))

;;; ----- entferne -----
; Wird ueber das remove hinaus gebraucht, um zu verhindern,
; dass das erste Element -die Containergrsse- entfernt wird.
(define
  (entferne stueck nummer container-liste)
  (cond
    ((null? container-liste) (error "falscher index!"))
    ((negative? nummer) (error "falscher index!"))
    ((zero? nummer)
     (cons
      (cons
       (caar container-liste) ; Fassungsvermoegen
       (remove stueck (cdar container-liste))) ; Inhalte
      (cdr container-liste)))
    (else
     (cons
      (car container-liste)
      (entferne stueck (sub1 nummer) (cdr container-liste))))))

;;; ----- fuelle -----
; Wird ueber das cons hinaus gebraucht, um zu verhindern, dass das 
; stueck als erstes Element -die Containergrsse- verwendet wird.
(define
  (fuelle stueck nummer container-liste)
  (cond
    ((null? container-liste) (error "falscher index!"))
    ((negative? nummer) (error "falscher index!"))
    ((zero? nummer)
     (cons
      (cons
       (caar container-liste) ; Fassungsvermoegen
       (cons stueck (cdar container-liste))) ; Inhalte
      (cdr container-liste)))
    (else
     (cons
      (car container-liste)
      (fuelle stueck (sub1 nummer) (cdr container-liste))))))

;;; ===== Test =====
;(define
;  (Test-mutiere container-liste . n)
;  ;(bkpt container-liste n)
;  (cond
;   ((null? n)
;    (Test-mutiere container-liste 10))
;   ((zero? (car n)) container-liste)
;  (else
;   (Test-mutiere
;    (mutiere
;     (random (length container-liste))
;     (random (length container-liste))
;     container-liste)
;    (sub1 (car n))))))
;
;(Test-mutiere *Container*)
;;; ===== Test-reine-Mutation =====
; Testet bewertet mit einer Mutation
;(define
;  (Test-reine-Mutation container-liste alte-liste n)
;  ;(writeln n (bewerte container-liste) container-liste)
;  (cond
;    ((zero? n) container-liste)
;    ((< (bewerte container-liste) (bewerte alte-liste))
;     (Test-reine-Mutation
;      (mutiere
;       (random (length alte-liste))
;       (random (length alte-liste))
;       alte-liste)
;      alte-liste
;      (sub1 n)))
;    (else
;     (Test-reine-Mutation
;      (mutiere
;       (random (length container-liste))
;       (random (length container-liste))
;       container-liste)
;      container-liste
;      (sub1 n)))))
;
;(Test-reine-Mutation *Container* *Container* 10000)
